/*
 *  OrthographicCamera.h
 */

#ifndef __ORTHOGRAPHIC_H
#define __ORTHOGRAPHIC_H

#include "Utils/globals.h"
#include "LinearAlgebra/Vector2.h"
#include "LinearAlgebra/Vector3.h"
#include "Camera/Camera.h"
#include "RayTracing/Ray.h"
#include "Geometry/Object.h"


/** This class represents an orthographic camera  */
template<int D>
class OrthographicCamera: public Camera<D>
{
protected:
	///Object associated to the camera
	VectorN<D> dir, dir_T;
	VectorN<D> center;
	Real h,w; 
	void setup()
	{ 
		if (D==2)
		{
			dir_T[0] = dir[1];
			dir_T[1] = -dir[0];
		}
		else
		{
			dir_T[0] = 1.;
			dir_T[1] = 1.;
			dir_T[2] = 0.;
			//dir_T.normalize();
		}

	}
public:
	
	/** Build the camera from viewing information.
	 */
	OrthographicCamera(VectorN<D> _center, VectorN<D> _dir, Real _w, Real _h=0.):center(_center),dir(_dir), w(_w), h(_h) { setup();}
	/// Calculates the ray
	virtual const VectorN<D> get_ray_direction(const VectorN<D-1>& p) const
	{
		return dir;
	}
	
	virtual Ray<D> get_ray(const VectorN<D-1>& p) const;
	/*{
		//TODO: Make it generic for 2D and 3D
		return Ray<D>(center, dir);
	}*/
};

template<>
Ray<2> OrthographicCamera<2>::get_ray(const VectorN<1>& p) const
{
	return Ray<2>(center + p[0]*dir_T*w, dir);
}
template<>
Ray<3> OrthographicCamera<3>::get_ray(const VectorN<2>& p) const
{
	Ray<3> nr(center + Vector3(p[0]*w, p[1]*h,0)*dir_T, dir);
	//printf("pos = %f %f %f, dir = %f %f %f\n", nr.get_origin()[0], nr.get_origin()[1], nr.get_origin()[2], nr.get_direction()[0], nr.get_direction()[1], nr.get_direction()[2]);
	return nr;
}

typedef OrthographicCamera<2> OrthographicCamera2D;
typedef OrthographicCamera<3> OrthographicCamera3D;

#endif

