#ifndef _EMISSION_H_
#define _EMISSION_H_

#include "Material/Material.h"
#include "RayTracing/Ray.h"
#include "RayTracing/Intersection.h"
#include "image/image.h"

#include <vector>

using namespace std;

/**	Class defining the simplest emissing
	material. 
	TODO: Parametrize the light distribution,
		now it's only gaussian. */
template<int D>
class Emission: public Material<D>
{
	Spectrum m_power;
	Imaging::Image *m_emission_map;
public:
	Emission(): Material<D>(Reflectance::EMISSIVE),m_power(1.){}
	Emission(const char* name):Material<D>(Reflectance::EMISSIVE),m_power(1.), m_emission_map(0)
	{
		m_emission_map = new Imaging::Image(name);
	}
	Emission( Spectrum power, const char* name ): Material<D>(Reflectance::EMISSIVE),m_power(power),m_emission_map(0)
	{
		m_emission_map = new Imaging::Image(name);
	}
	~Emission()
	{	if(m_emission_map) delete m_emission_map;	}

	
	Spectrum f(const VectorN<D> &omega_o, const VectorN<D> &omega_i, const VectorN<D> &normal, const Vector2 &uv) const
	{
		
		return 0.;

	}

	/** Computes the pdf */
	Real p(const VectorN<D> &omega_o, const VectorN<D> &omega_i, const VectorN<D> &normal, const Vector2 &uv )const
	{	return 1.; }

	Spectrum sample_direction(const VectorN<D> &omega_o, VectorN<D> &omega_i, const VectorN<D> &normal, const Vector2 &uv, Real &pdf) const
	{
		pdf = 1.;
		// Need for importance sampling...	
		return Spectrum(0.);
	}
	Spectrum sample_outgoing_ray(const Intersection<D> &it, Ray<D> &new_ray, Real &pdf) const
	{
		pdf = 1.;
		return Spectrum(0.);
	}

	// TIME RESOLVED MATERIAL FUNCTIONS
	/** Computes the local direct reflected (or refracted) light. 
		omega_o is the view (incoming) direction, while omega_i is light (outgoing)*/
	virtual Spectrum f( const Spectrum &L_i, const VectorN<D> &omega_o, const VectorN<D> &omega_i, const VectorN<D> &normal, const Vector2 &uv, Real &delta_time, Real &pdf ) const 
	{	
		pdf = 1; delta_time = 0;
		return f(omega_o,omega_i,normal,uv);
	}

	Spectrum emit_light( const VectorN<D> &omega_i, const VectorN<D> &normal, const Vector2 &uv )const
	{
		Real di = dot_clamped(normal, omega_i);
		Spectrum p = m_power * di;

		if( m_emission_map )
		{
			Imaging::RGBColor rgb = (*m_emission_map)(m_emission_map->width()*uv[0],m_emission_map->height()*uv[1]);
			//printf("RGB Emission @[%f,%f]: %f, %f, %f\n", m_emission_map->width()*uv[0],m_emission_map->height()*uv[1], rgb.r(), rgb.g(), rgb.b());
			p *= Spectrum(rgb.r(), rgb.g(), rgb.b());
		}
		
		return p;	
	}

	Spectrum emit_light( const VectorN<D> &omega_i, const VectorN<D> &normal, const Vector2 &uv, Real &delta_time, Real &pdf )const
	{	pdf = 1; delta_time = 0; return emit_light( omega_i, normal, uv); }

	Spectrum get_absorption( const Vector2 &uv ) const
	{
		return Spectrum(1.);
	}
}; //Emission

typedef Emission<3> Emission3D;
typedef Emission<2> Emission2D;

#endif //_EMISSION_H_