#ifndef _TEXTURE_H_
#define _TEXTURE_H_

#include "Utils/globals.h"
#include "LinearAlgebra/Vector3.h"

class Texture
{
public:
	enum Channels {Luminance=1, RGB=3, RGBA=4} ;
	enum SizeElement {Byte=1, Float=4} ;
	enum Filtering {Exact, NearestNeighbor, Bilinear, Cubic};
	enum TextureWrap {Black, Clamp, Repeat, InverseRepeat};

protected:
	unsigned int m_width, m_height;
	Real m_inv_width, m_inv_height;
	Channels m_nb_channels;
	SizeElement m_size_element;
	Filtering m_spatial_filtering;
	TextureWrap m_wrap_s, m_wrap_t;
public:
	Texture()
		: m_width(0), m_height(0), m_inv_width(0.), m_inv_height(0.), m_nb_channels(RGB), m_size_element(Float), 
		m_spatial_filtering(Bilinear), m_wrap_s(Black), m_wrap_t(Black){}

	Texture(unsigned int w, unsigned int h, Channels nb_channels = RGB, SizeElement size = Float,
		Filtering magnification = Bilinear, Filtering minification = Bilinear )
		: m_width(w), m_height(h), m_inv_width(1./(Real)m_width), m_inv_height(1./(Real)m_height), 
		  m_nb_channels(nb_channels), m_size_element(size), m_spatial_filtering(magnification), 
		  m_wrap_s(Black), m_wrap_t(Black){}

	
	unsigned int get_width()const{return m_width;}
	unsigned int get_height()const{return m_height;}

	//Resize (and potentionally reshape) the texture
	virtual void resize(int w, int h);
	// Access data
	virtual const Spectrum &operator()(Real x, Real y)const = 0;
	inline const Spectrum &operator()(const Vector2 &v)const{return this->operator()(v[0],v[1]);}
	virtual const Spectrum &operator()(int x, int y)const = 0;
}; //Texture


#endif //_TEXTURE_H_